@@ -5,9 +5,9 @@ use strict;
 use warnings;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2023-02-10'; # DATE
+our $DATE = '2023-02-14'; # DATE
 our $DIST = 'App-CSVUtils'; # DIST
-our $VERSION = '1.009'; # VERSION
+our $VERSION = '1.010'; # VERSION
 
 use App::CSVUtils qw(
                         gen_csv_util
@@ -83,7 +83,7 @@ App::CSVUtils::csv_munge_field - Munge a field in every row of CSV file with Per
 
 =head1 VERSION
 
-This document describes version 1.009 of App::CSVUtils::csv_munge_field (from Perl distribution App-CSVUtils), released on 2023-02-10.
+This document describes version 1.010 of App::CSVUtils::csv_munge_field (from Perl distribution App-CSVUtils), released on 2023-02-14.
 
 =head1 FUNCTIONS
 
@@ -128,6 +128,41 @@ Perl code.
 
 Field name.
 
+=item * B<inplace> => I<true>
+
+Output to the same file as input.
+
+Normally, you output to a different file than input. If you try to output to the
+same file (C<-o INPUT.csv -O>) you will clobber the input file; thus the utility
+prevents you from doing it. However, with this C<--inplace> option, you can
+output to the same file. Like perl's C<-i> option, this will first output to a
+temporary file in the same directory as the input file then rename to the final
+file at the end. You cannot specify output file (C<-o>) when using this option,
+but you can specify backup extension with C<-b> option.
+
+Some caveats:
+
+=over
+
+=item * if input file is a symbolic link, it will be replaced with a regular file;
+
+=item * renaming (implemented using C<rename()>) can fail if input filename is too long;
+
+=item * value specified in C<--inplace-backup-ext> is currently not checked for
+acceptable characters;
+
+=item * things can also fail if permissions are restrictive;
+
+=back
+
+=item * B<inplace_backup_ext> => I<str> (default: "")
+
+Extension to add for backup of input file.
+
+In inplace mode (C<--inplace>), if this option is set to a non-empty string, will
+rename the input file using this extension as a backup. The old existing backup
+will be overwritten, if any.
+
 =item * B<input_escape_char> => I<str>
 
 Specify character to escape value in field in input CSV, will be passed to Text::CSV_XS.
